#
# SliMP3 Server
# Copyright (C) 2001 Sean Adams, Slim Devices Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.



#
# The MAS3507D i2c protocol is quite ugly. We need four different formats depending on whether
# the data to be read/written is a "register", one of two "data banks", or a "program offset".
#
# When this file is included, it builds %mas, which contains the addresses of all 
# interesting registers in the MAS3507D. See the MAS3507D data sheet for an explantion
# of each of these functions.
#

$maslist='
# 
# MAS3507D memory and registers
#
# r   = register
# d0  = d0 memory
# d1  = d1 memory
# run = run command
#

r 8e dccf
r aa mute
r ed piodata
r e6 config
r e7 kprescale
r 6b kbass
r 6f ktreble

d0 0300 mpegframecount
d0 0301 mpegstatus1
d0 0302 mpegstatus2
d0 0303 crcerrorcount
d0 0304 numberofancillarybits
d0 0305 ancillary data

d0 0305 ancillarydata

d0 032d plloffset48   # NOTE: DATA SHEET IS INCORRECT!!!
d0 032e plloffset44   # These addresses are listed in the data sheet as 36x
d0 032f outputconfig

d1 07f8 ll   # gain controls
d1 07f9 lr
d1 07fa rl
d1 07fb rr

d1 0ff6 version
d1 0ff7 designcode
d1 0ff8 idunno
d1 0ff9 desc1
d1 0ffa desc2
d1 0ffb desc3
d1 0ffc desc4
d1 0ffd desc5
d1 0ffe desc6
d1 0fff desc7   

run 0fcd loadconfig
run 0475 setpll

'; # END OF MAS REGISTER LISTING

my ($line, $bank, $address, $label);
foreach $line (split /\n/, $maslist) {
  $line=~s/#.*//;
  ($bank, $address, $label)=split ' ', $line;
  $mas{"$label.bank"}=$bank;
  $mas{"$label.address"}=$address;
  }

1;

#--------------------------------------------

sub maswrite {              # Generate the i2c sequence for writing to the MAS3507D
  my ($key, $data)=@_;
  my (@a,@d,$i2c,$bank,$address);
  die "Unknown MAS3507D setting: $key\n" unless ($mas{"$key.address"});
  $bank=$mas{"$key.bank"};
  $address=$mas{"$key.address"};

  $data=~s/[^0-9a-fA-F]//g;

  # Turn address and data into an array of nibbles with least significant at [0]
  while ((length($address))>0) {
    push @a, chop($address); 
    }  
  while (length($data)>0) {
    push @d, chop($data); 
    }  

  # See MAS3507D data sheet for exaplanation of their i2c functions
  if ($bank eq 'r') {
    $i2c="s 3a 68 9$a[1] $a[0]$d[0] $d[4]$d[3] $d[2]$d[1] p";
    }
  elsif ($bank eq 'd0') {
    $i2c="s 3a 68 a0 00 00 01 $a[3]$a[2] $a[1]$a[0] $d[3]$d[2] $d[1]$d[0] 00 0$d[4] p";
    }
  elsif ($bank eq 'd1') {
    $i2c="s 3a 68 b0 00 00 01 $a[3]$a[2] $a[1]$a[0] $d[3]$d[2] $d[1]$d[0] 00 0$d[4] p";
    }
  elsif ($bank eq 'run') {
    $i2c="s 3a 68 $a[3]$a[2] $a[1]$a[0] p";
    }
  $i2c=~s/([\dA-Fa-f][\dA-Fa-f])/'w'.pack ("C", hex ($1))/eg;  # pack hex values and add write, ack commands
  $i2c=~s/ //g;
  $i2c;
  }

sub masread {
  my ($key)=@_;
  my (@a,$i2c,$bank,$address);
  die "Unknown MAS3507D setting: $key\n" unless ($mas{"$key.address"});
  $bank=$mas{"$key.bank"};
  $address=$mas{"$key.address"};

  while (length($address)>0) {
    push @a, chop($address); 
    }  

  if ($bank eq 'r') {
    $i2c="s 3a 68 d$a[1] $a[0]0 p s 3a 69 s 3b ra ra ra rn p";  # reads two bytes
    }
  elsif ($bank eq 'd0') {
    $i2c="s 3a 68 e0 00 00 01 $a[3]$a[2] $a[1]$a[0] p s 3a 69 s 3b ra ra ra rn p" ; # 8 bytes, 5 sig nibbles
    }
  elsif ($bank eq 'd1') {
    $i2c="s 3a 68 f0 00 00 01 $a[3]$a[2] $a[1]$a[0] p s 3a 69 s 3b ra ra ra rn p";
    }
  elsif ($bank eq 'run') {
    die "Invalid MAS3507D operation: read run offset??";
    }
  $i2c=~s/([\dA-Fa-f][\dA-Fa-f])/'w'.pack ("C", hex ($1))/eg;  # pack hex values and add write, ack commands
  $i2c;
  }

