#
# SliMP3 Server
# Copyright (C) 2001 Sean Adams, Slim Devices Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.



#
# Timers are stored as a struct containing the time after which the timer
# should fire, a reference to the function, and a list of arguments.
# Timers are checked whenever we come out of select, or every 10ms,
# whichever is greater.

struct( timer_struct => [
  when   =>'$',   # when to fire, epoch seconds
  client =>'$',   # refence to the client struct
  subptr =>'$',   # reference to the function
  args   =>'@'    # function arguments
]);


sub checktimers {
  my ($timer);
  while ($timers[0] && (($timers[0]->when) <= time())) {
    $timer = shift(@timers);
    $subptr = $timer->subptr;
    $c = $timer->client;
    &$subptr;
  }
}

sub listtimers {
  my ($timer);
  print "timers: \n";
  foreach $timer (@timers) {
    print join("\t", $timer->client, $timer->when, $timer->subptr), "\n";
  }
}


# The list of timers is maintained in sorted order, so
# we only have to check the head.
sub settimer {
  my ($client, $when, $subptr, @args)=@_;
  print "subptr: $subptr\n";
  for ($i=0; $i<(@timers); $i++) {
    last if ($timers[$i]->when > $when);
  }
  my($newtimer) = new timer_struct;
  $newtimer->client($client);
  $newtimer->when($when);
  $newtimer->subptr($subptr);
  $newtimer->args(@args);
  splice(@timers, $i, 0, $newtimer);
  &listtimers;
}



$timer_updateclock = sub {
  print "timer_updateclock\n";
  $date=`date`;
  &vfd_update($date, '');
  &settimer($c, (time()+1), $timer_updateclock);
};

sub alarmclock_on {
  print "alarmclock_on\n";
  $c->mode_clock(1);
  $subptr=$timer_updateclock;
  &$subptr;
}

sub alarmclock_off{
  $c->mode_clock(0);
  &cleartimer($c, $timer_updateclock);
}

1;

