#
# SliMP3 Server
# Copyright (C) 2001 Sean Adams, Slim Devices Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


#  
# vfd.pl initiliazion:  Builds %vfdcmd, an associative array containing the 
# packed codes for all the noritake VFD commands

$vfd_commands='

CLEAR  00000001  ; Clear display. Wait 2.5ms after doing this
    
HOME   0000001X  ; Cursor home
    
DECSC  00000100  ; mode: decrement address, shift cursor
DECSD  00000101  ; mode: decrement address, shift display
INCSC  00000110  ; mode: increment address, shift cursor 
INCSD  00000111  ; mode: increment address, shift display
    
OFF    000010XX  ; display off
CUR    00001110  ; display on, with cursor
BLK    00001101  ; display on, with blinking
CURBLK 00001111  ; display on, cursor and blinking
    
MOVEL  000100XX  ; move cursor left
MOVER  000101XX  ; move cursor right
SHIFTL 000110XX  ; shift display left
SHIFTR 000111XX  ; shift display right
    
FSET8  0011XXXX  ; set 8-bit mode
FSET4  0010XXXX  ; set 4-bit mode (dont do this!)
    
; brightness is not a command. It is sent as a character byte, right after FSET8
    
BR25   XXXXXX11  ; 25% brightness
BR50   XXXXXX10  ; 50%
BR75   XXXXXX01  ; 75%
BR100  XXXXXX00  ; 100%

; set CG ram address
;
;  01AAAAAA   where AAAAAA is the 6-bit address to set   
    
; set DD ram address
;
;  1AAAAAAA where AAAAAAA is the 7-bit address
';
    
$vfdcode_wait=pack 'B8', '00000000';       # these codes identify the operation to 
$vfdcode_cmd= pack 'B8', '00000010';       # perform on each byte of data sent.
$vfdcode_char=pack 'B8', '00000011'; 
$vfdw=$vfdcode_wait.pack 'B8', '11111111'; # Example: this is a 255ms delay

foreach $line (split /\n/, $vfd_commands) {
  $line=~/(\S+)\s+([X0-9]+)/ || next;
  $command=$1;
  $binary=$2;
  $binary=~s/X/0/g;
  $vfdcmd{$command}=pack 'B8', $binary;
}

1;

#
# Adjust display brightness by delta (-3 to +3)
#
sub vfd_brightness_adj {
  my ($delta)=@_;
  $c->vfdbrightness($c->vfdbrightness+$delta);
  $c->vfdbrightness(1) if ($c->vfdbrightness<1);
  $c->vfdbrightness(4) if ($c->vfdbrightness>4);
  &vfd_brightness_set($c->vfdbrightness);
  }

#
# Set vfd brightness (1 to 4)
#    
sub vfd_brightness_set {
  my ($vfdbrightness)=@_;
  my $brcode;   
  $d_u && print "Setting brightness: $vfdbrightness\n";
  if ($vfdbrightness==1) {
    $brcode=$vfdcmd{'BR25'};   
  } elsif ($vfdbrightness==2) {
    $brcode=$vfdcmd{'BR50'};
  } elsif ($vfdbrightness==3) {
    $brcode=$vfdcmd{'BR75'};
  } elsif ($vfdbrightness==4) {
    $brcode=$vfdcmd{'BR100'};
    }
  &vfd_send('_FSET8',$brcode);
  }

#
# completely refresh the display
#
sub vfd_refresh {
  $temp1=$c->prevline1;
  $temp2=$c->prevline2;
  $c->prevline1(' ' x 40);
  $c->prevline2(' ' x 40);
  &vfd_update($temp1, $temp2);
}

#
# Update the client's display - just the characters that have changed
#
sub vfd_update {
  my ($line1,$line2)=@_;
  while (length($line1) < length($c->prevline1)) {
    $line1.=' ';
    }
  while (length($line2) < length($c->prevline2)) {
    $line2.=' ';
    }
     
  while (length($line1)>40) {
    chop $line1;
    }
  while (length($line2)>40) {
    chop $line2;
    }
     
  $c->prevline1($line1);
  $c->prevline2($line2);
  
  &vfd_send('_HOME',$line1,'_HOME2',$line2);
  
  if ($d_d) {
    print " ---------------------------------------- \n",
          "|$line1", ' ' x (40-length($line1)), "|\n",
          "|$line2", ' ' x (40-length($line2)), "|\n",
          " ----------------------------------------\n";
    }
  }

#
# Converts a list of human readable VFD commands/chars into data that is sent to the player
#
sub vfd_send {
  my $vfddata, $arg, $line, $char, @chars, $header;
  foreach $arg (@_) {
    $line=$arg;
    $line=~s/\n//;
    @chars=();
    if ($line eq '_HOME2') {
      $vfddata.=$vfdcode_cmd.pack 'B8', '11000000';
      }
    elsif ($line=~/^_BR(.+)/) {
      $vfddata.=$vfdcode_cmd.$vfdcmd{'FSET8'}.$code_char.$lcdcmd{"BR$1"};
      if ($delay>0) {
       $vfddata.=$vfdcode_wait.pack 'C', $delay;
       }
      } 
    elsif ($line=~/^_(.+)/) {
      $vfddata.=$vfdcode_cmd.$vfdcmd{$1};
      }
    elsif ($line=~/^\.(.+)/) {
      $vfddata.=$vfdcode_wait.pack 'C', $1;
      }
    elsif ($line=~/^\*(.+)/) {
      $delay=$1;
      }
    else {
      while (length($line)>0) {
        $char=chop($line);
        push @chars, $char;
        }
      while ((@chars)>0) {
        $char=pop(@chars);
        if ($delay>0) {
          $vfddata.=$vfdcode_wait.pack 'C', $delay;
          }
        $vfddata.=$vfdcode_char.$char;
        }
      }
    }
  $header="l                 ";
  send $udpsock, $header.$vfddata, 0, $clientpaddr;
  }

