#!/usr/bin/perl
#
# SliMP3 Server
# Copyright (C) 2001 Sean Adams, Slim Devices Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


#-------- You probably don't want to change any of these ---------#

$os='auto';           # Usually 'auto' - can override with 'unix', 'win', or 'mac'
$serverport=1069;     # Port that the server should listen on
$chunksize=1400;      # Reduce this if your path MTU is small
$irmintime=60000;     # If time between IR commands > $irmintime, then the code is considered a new button press
$maxbuf=2**16;        # size of player's SRAM buffer: 1Mbit == 2^16 x 16-bits
$MASfreq=14.31818;    # frequency in MHZ of the MAS3507D

#-----------------------------------------------------------------#

use Config;
use Class::Struct;
use IO::Socket;
use Time::localtime;
use FileHandle;

require 'lib-slimp3/osdetect.pl';
require $prefs_file;
require 'lib-slimp3/mas3507d.pl';
require 'lib-slimp3/sta013.pl';
require 'lib-slimp3/cs4341.pl';
require 'lib-slimp3/vfd.pl';
require 'lib-slimp3/ir.pl';
require 'lib-slimp3/buttons_browse.pl';
require 'lib-slimp3/timers.pl';

#
# The following struct contains all the state that we keep about each player.
# Each time we receive a packet for a particular client, we set the global $c
# to point to the struct object for this client.
#
# FIXME:
# Currently, we identify a client by it's IP address and source port. The 
# problem with this is that if the client is behind NAT, his UDP source port
# may change at any time. In the future, we'll have the client send a unique
# code with each request packet, and we'll allow his source port to change.

struct( client_state_struct => [
  paddr          =>'$',  # ip and port
  mode_random    =>'$',
  mode_play      =>'$',
  mode_pause     =>'$',
  mode_sleep     =>'$',  
  mode_clock     =>'$',
  prevwptr       =>'$',  # write pointer into the client's buffer at prev req packet
  lastirtime     =>'$',  # time at which we last received an IR code
  lastircode     =>'$',
  pwd            =>'$',  # present directory
  selectedisnext =>'$',  # play the selected track when current track ends, instead of the next one in the list
  curdiritem     =>'$',   
  rand_basedir   =>'$',
  rand_file      =>'$',
  mp3filehandle  =>'$',  # the currently open MP3 file
  numirrepeats   =>'$',
  waitforstart   =>'$',
  chunk          =>'$',
  numdiritem     =>'$',
  diritems       =>'@',  # list of file names in the current directory

  vfdbrightness  =>'$',
  prevline1      =>'$',
  prevline2      =>'$',  

]);

#
# initialization
#
    
srand;

$udpsock=IO::Socket::INET->new(
  Proto => 'udp',
#  LocalHost=>INADDR_ANY,
  LocalPort=>$serverport,
  ) or die "socket: $@";

#
# main loop
#

while (1) {
  $clientpaddr=recv($udpsock,$msg,1500,0);
  $c = &getclient($clientpaddr);
  &processmsg($msg);
  &checktimers;
}


#----------------------

sub getclient {
  my ($clientpaddr)=@_;
  return $clients{$clientpaddr} if ($clients{$clientpaddr});

#if we haven't seen this client, initialialize a new one
#
  print "New client: \n";
  $c = $clients{$clientpaddr} = new client_state_struct;  
  $c->prevwptr(-1);
  $c->pwd('');
  $c->mp3filehandle(new FileHandle);
  $c->vfdbrightness(4);
  
  &loaddir('');
  &vfd_refresh;
  return $c;
}


sub opennext {
  my $fullpath, $f, $randpath;
  if ($c->mode_random==0) {
    # sequential play
    unless ($c->selectedisnext) {
      if ($c->curdiritem==($c->numdiritem-1)) {
        $c->curdiritem(0);
        } else {
        $c->curdiritem($c->curdiritem+1);
        }
      }
    $f=$c->diritems($c->curdiritem);
    $fullpath=$mp3dir.(($c->pwd ne '') ? $p : '').$c->pwd.$p.$f;
    if (&ismp3($f)) {
      $d_f && print "opening next: $fullpath\n";
    } else {
      $d_f && print "next item is a directory... stopping\n";
      &mp3_stop;
    }
  } else {
    # random mode
    $randpath=&randmp3("$mp3dir$p".$c->rand_basedir);
    $d_f && print "opening random: $randpath\n";
    $fullpath="$randpath";
  }
  &showdir;
  $c->mp3filehandle->open($fullpath) || die "couldn't open: $fullpath";
  binmode($c->mp3filehandle);
  }

# FIXME: need to scan the tree in advance, so we don't favor files in smaller
# branches, or play the same song repeatedly
sub randmp3 {   # recursive subroutine to find a random mp3 file
  my($base) =@_;
  my(@list,$item,$path,$recurs,$i,$j,$found);
  $base=~s/$p$//; # remove trailing slash if present
  $d_f && print "Searching: $base\n" if ($randdebug);
  opendir BASE,$base;
  while ($item=readdir(BASE)) {
    next if ($item eq '.');
    next if ($item eq '..');
    push @list,$item;
    }
  @list[-$i,$j]=@list[$j,-$i] while $j = rand(@list - $i), ++$i < @list; # Fisher-Yates randomize
  while ((!$found) && ($item=pop(@list))) {
    $path="$base$p$item";
    $d_f && print "Testing $path\n";
    if (-f$path) {
      $found=$path; 
      $c->rand_file($item);
      $d_f && print "Found: $found\n";
      }
    if (-d$path) {
      $recurs=&randmp3($path);
      $found=$recurs if ($recurs) # if directory contains an MP3 file, return path to that mp3
      }
    }
  return $found;  # returns null if nothing found
  }

sub random_start {
  $d_u && print "Random mode\n";
}

sub mp3_streamcontrol {
  my ($cmd)=@_;
  my %code = ('start',  pack('C', 1),
              'pause',  pack('C', 2),
              'resume', pack('C', 4));
  my $code = $code{$cmd};
  send $udpsock, 's'.$code, 0, $clientpaddr; 
}

sub mp3_stop {
  $c->mode_play(0);
  mp3_streamcontrol('pause');
}

sub mp3_pause {
  if ($c->mode_pause && $c->mode_play) {
    &mp3_streamcontrol('resume');
    $c->mode_pause(0);
  } else {
    &mp3_streamcontrol('pause');
    $c->mode_pause(1);
  }
}

sub mp3_start {
  local ($f) =@_;
  $c->mode_play(1);
  $c->mode_pause(0);

  send ($udpsock, '2                 '
    .&cswrite('MCLK',   '00')
    .&cswrite('Mode',   '00')
    .&cswrite('Mixing', '49')
    .&cswrite('VolA',   '00')
    .&cswrite('VolB',   '00')
    .&maswrite('config','00002')
    .&maswrite('loadconfig')
    .&maswrite('plloffset48','5d9e9')
    .&maswrite('plloffset44','cecf5')
    .&maswrite('setpll')
    .&csread('Mode')
    , 0, $clientpaddr);
  select undef,undef,undef,.05;
  &mp3_streamcontrol('start');
  $c->waitforstart(1);
  }

sub processmsg {
  local($msg)=@_;
  @octets=split(//,$msg);
  $type=unpack('a',$octets[0]);

  if ($type eq 'i') {
    if ($d_u) { 
      print "ir: \"";
      for ($oi=0;$oi<=11;$oi++) {
        $d_u && print unpack('H2',$octets[$oi]), ' ';
      }
      print "\"\n";
    }
    $ircode=$ircodes{unpack('H2',$octets[11])};
    $irtime=unpack('N',$octets[2].$octets[3].$octets[4].$octets[5]);
    $irtimedif=$irtime - $c->lastirtime;
    $c->lastirtime($irtime);
    $d_u && print "ircode=$ircode timer=$irtime timedif=$irtimedif last=".$c->lastircode." numrep=".$c->numirrepeats."\n";
    if (($ircode eq $c->lastircode)&&($irtimedif<$irmintime)) {
      &irrepeat;
      } else {
      $c->numirrepeats(0);
      &processir($ircode);
      }
  } elsif ($type eq 'r') {
    &gotrequest;
  } else {
    $d_u && print "unknown type: $type\n";
    }
  }

# This is all there is to it - it's like tftp, except that the client requests each
# chunk, as opposed to acknowlegding the data as we send it. Client handles timeouts.
sub gotrequest {
  my $wptrhp, $wptrlp, $wptrh, $wptrl, $wptrhh, $wptrlh, $wptr, $header, $chunk;
  $wptrhp=$octets[2];
  $wptrlp=$octets[3];
  $wptrh=unpack 'C', $wptrhp;
  $wptrl=unpack 'C', $wptrlp;
  $wptrhh=unpack 'H2', $wptrhp;
  $wptrlh=unpack 'H2', $wptrlp;
  $wptr=$wptrh*256+$wptrl;
  $d_pv && print "Request wptr=$wptrhh.$wptrlh\n";
  if ($c->waitforstart==1) {
    if ($wptr==0) {
      $c->waitforstart(0);
      } else {
      $d_p && print "Ignoring request from previous stream\n";
      return;
      }
    }
  if (!$c->mode_play) {
#    $d_u && print "Stopping\n";
    return;
    }
  if ($c->prevwptr==$wptr) {
    $d_p && print "Duplicate request: wptr=$wptr\n";
    } else {
    read ($c->mp3filehandle, $chunk, $chunksize);
    $c->chunk($chunk);
    }  
  if (length($c->chunk)==0) {
    &opennext;
    $c->selectedisnext(0);
    read ($c->mp3filehandle, $chunk, $chunksize);
    $c->chunk($chunk);
    } else {
    $header='m '.$wptrhp.$wptrlp.'              ';
    $c->chunk($c->chunk.'.') if ((length($c->chunk)%2)!=0);
    send $udpsock, $header.$c->chunk, 0, $clientpaddr;
    $d_pv && print "Sending ".length($c->chunk)." bytes. wptr=$wptrhh.$wptrlh\n";
    }
  $c->prevwptr($wptr);
  }

sub irrepeat {
  $c->numirrepeats($c->numirrepeats+1);
  if ($c->numirrepeats>5) {
    if (($c->lastircode eq 'down') || ($c->lastircode eq 'up')) {
      &processir($c->lastircode) 
    }
  }
}

sub processir {
  $d_u && print "processir begin: $c\n";
  ($ircode)=@_;
  $c->lastircode($ircode);  
  $d_u && print "ir: $ircode\n"; 
  if ($subref=$buttons_browse{$ircode}) {
    &$subref;
  } else {
    $d_u && print "button not implemented in this mode: $ircode\n";
  }
  &showdir();
}

sub showdir {
  my $line1, $line2, $shortpwd, $rand_shortpath, $pwd;
  if (!($c->mode_random)) {
    $pwd=$c->pwd;
    if ($pwd eq '') {
      $line1="Top level (".$c->numdiritem.")";
    } else {
      $shortpwd=$c->pwd;
      while ($shortpwd=~s/^.+($p.+$p.+)/$1/) { ; }  # show only deepest two levels
      $shortpwd=~s/^$p//;                             # remove leading slash
      $line1="$shortpwd (".$c->numdiritem.")";
    }    

    if ($c->pwd eq '') {
      $lastselection{'Top level'}=$c->curdiritem;
    } else {
      $lastselection{$c->pwd}=$c->curdiritem;
    }
    $line2=$c->diritems($c->curdiritem);
  } else {    # random mode
    $c->rand_basedir=~/([^$p]+)$/;
    $rand_shortpath=$1;
    $rand_shortpath='All files' unless ($rand_shortpath=~/./);
    $line1="Random: $rand_shortpath";
    $line2=$c->rand_file;
  }
  &vfd_update($line1, $line2);
}

sub isdir {
  my($f) = @_;
  my $dirpath = $mp3dir.(($c->pwd eq '')? '' : $p.$c->pwd).$p.$f;
  $d_f && print "isdir($dirpath) == ".(1 * -d$dirpath)."\n";
  return(-d$dirpath);
}

sub ismp3 {
# Maybe add more extensive checks here to ensure it is an mp3 file
  my($f)=@_;
  my $fullpath=$mp3dir.(($c->pwd eq '')? '' : $p.$c->pwd).$p.$f;
  $fullpath=~/\.rip$/ && return 0;
  $fullpath=~/\.wav$/ && return 0;
  $fullpath=~/\.aiff$/ && return 0;
  $d_f && print "ismp3($fullpath) == ".(1 * -f$fullpath)."\n";
  return(-f$fullpath);
  }

sub loaddir {
  my($dir)=@_;
  my $pwd, $dirpath, $ls, $i, $diritem;

  $pwd=$c->pwd;
  if ($dir eq '') {
    $pwd='';
  } elsif ($dir eq '..') {
    $pwd=~s/$p?[^$p]+$//;
  } else {
    $pwd.=$p unless ($pwd eq '');
    $pwd.=$dir;
  }
  if ($pwd eq '') {
    $dirpath=$mp3dir;
    } else {
    $dirpath="$mp3dir$p$pwd"; 
  }
  die "No such dir: $dirpath" unless (-d"$dirpath");
  $d_f && print "Opening dir: $dirpath\n";
  opendir DIR, "$dirpath" || die "opendir: $dirpath";
  $i=0;
  foreach $diritem (sort(readdir(DIR))) {
    next if ($diritem=~/^\./);
    next if (($os eq 'mac') && ($diritem=~/Icon/));
    $d_f && print "$diritem\n";
    $c->diritems($i++, $diritem);
  }
  $c->numdiritem($i);
  if ($pwd eq '') {
    $ls='Top level';
  } else {
    $ls=$pwd;
  }
  $d_f && print "lastselection{$ls}==".$lastselection{$ls}."\n";
  if (($lastselection{$ls}) && ($lastselection{$ls}< $c->numdiritem)) {
    $c->curdiritem($lastselection{$ls});
  } else {
    $c->curdiritem(0);
  }
  $c->pwd($pwd);
}


